@@ -1,37 +1,38 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import collections.abc
+from collections.abc import Callable
+from collections.abc import Collection
+from collections.abc import Iterable
+from collections.abc import Iterator
+from collections.abc import Mapping
+from collections.abc import MutableMapping
+from collections.abc import Sequence
+import dataclasses
+import enum
 import inspect
-import warnings
 from typing import Any
-from typing import Callable
-from typing import Collection
-from typing import Iterable
-from typing import Iterator
-from typing import List
-from typing import Mapping
-from typing import MutableMapping
+from typing import final
 from typing import NamedTuple
-from typing import Optional
 from typing import overload
-from typing import Sequence
-from typing import Set
-from typing import Tuple
-from typing import Type
 from typing import TYPE_CHECKING
 from typing import TypeVar
 from typing import Union
-
-import attr
+import warnings
 
 from .._code import getfslineno
-from ..compat import ascii_escaped
-from ..compat import final
 from ..compat import NOTSET
 from ..compat import NotSetType
 from _pytest.config import Config
 from _pytest.deprecated import check_ispytest
+from _pytest.deprecated import MARKED_FIXTURE
 from _pytest.outcomes import fail
+from _pytest.raises import AbstractRaises
+from _pytest.scope import _ScopeName
 from _pytest.warning_types import PytestUnknownMarkWarning
 
+
 if TYPE_CHECKING:
     from ..nodes import Node
 
@@ -39,33 +40,37 @@ if TYPE_CHECKING:
 EMPTY_PARAMETERSET_OPTION = "empty_parameter_set_mark"
 
 
+# Singleton type for HIDDEN_PARAM, as described in:
+# https://www.python.org/dev/peps/pep-0484/#support-for-singleton-types-in-unions
+class _HiddenParam(enum.Enum):
+    token = 0
+
+
+#: Can be used as a parameter set id to hide it from the test name.
+HIDDEN_PARAM = _HiddenParam.token
+
+
 def istestfunc(func) -> bool:
     return callable(func) and getattr(func, "__name__", "<lambda>") != "<lambda>"
 
 
 def get_empty_parameterset_mark(
     config: Config, argnames: Sequence[str], func
-) -> "MarkDecorator":
+) -> MarkDecorator:
     from ..nodes import Collector
 
-    fs, lineno = getfslineno(func)
-    reason = "got empty parameter set %r, function %s at %s:%d" % (
-        argnames,
-        func.__name__,
-        fs,
-        lineno,
-    )
+    argslisting = ", ".join(argnames)
 
+    fs, lineno = getfslineno(func)
+    reason = f"got empty parameter set for ({argslisting})"
     requested_mark = config.getini(EMPTY_PARAMETERSET_OPTION)
     if requested_mark in ("", None, "skip"):
         mark = MARK_GEN.skip(reason=reason)
     elif requested_mark == "xfail":
         mark = MARK_GEN.xfail(reason=reason, run=False)
     elif requested_mark == "fail_at_collect":
-        f_name = func.__name__
-        _, lineno = getfslineno(func)
         raise Collector.CollectError(
-            "Empty parameter set in '%s' at line %d" % (f_name, lineno + 1)
+            f"Empty parameter set in '{func.__name__}' at line {lineno + 1}"
         )
     else:
         raise LookupError(requested_mark)
@@ -73,34 +78,68 @@ def get_empty_parameterset_mark(
 
 
 class ParameterSet(NamedTuple):
-    values: Sequence[Union[object, NotSetType]]
-    marks: Collection[Union["MarkDecorator", "Mark"]]
-    id: Optional[str]
+    """A set of values for a set of parameters along with associated marks and
+    an optional ID for the set.
+
+    Examples::
+
+        pytest.param(1, 2, 3)
+        # ParameterSet(values=(1, 2, 3), marks=(), id=None)
+
+        pytest.param("hello", id="greeting")
+        # ParameterSet(values=("hello",), marks=(), id="greeting")
+
+        # Parameter set with marks
+        pytest.param(42, marks=pytest.mark.xfail)
+        # ParameterSet(values=(42,), marks=(MarkDecorator(...),), id=None)
+
+        # From parametrize mark (parameter names + list of parameter sets)
+        pytest.mark.parametrize(
+            ("a", "b", "expected"),
+            [
+                (1, 2, 3),
+                pytest.param(40, 2, 42, id="everything"),
+            ],
+        )
+        # ParameterSet(values=(1, 2, 3), marks=(), id=None)
+        # ParameterSet(values=(2, 2, 3), marks=(), id="everything")
+    """
+
+    values: Sequence[object | NotSetType]
+    marks: Collection[MarkDecorator | Mark]
+    id: str | _HiddenParam | None
 
     @classmethod
     def param(
         cls,
         *values: object,
-        marks: Union["MarkDecorator", Collection[Union["MarkDecorator", "Mark"]]] = (),
-        id: Optional[str] = None,
-    ) -> "ParameterSet":
+        marks: MarkDecorator | Collection[MarkDecorator | Mark] = (),
+        id: str | _HiddenParam | None = None,
+    ) -> ParameterSet:
         if isinstance(marks, MarkDecorator):
             marks = (marks,)
         else:
             assert isinstance(marks, collections.abc.Collection)
+        if any(i.name == "usefixtures" for i in marks):
+            raise ValueError(
+                "pytest.param cannot add pytest.mark.usefixtures; see "
+                "https://docs.pytest.org/en/stable/reference/reference.html#pytest-param"
+            )
 
         if id is not None:
-            if not isinstance(id, str):
-                raise TypeError(f"Expected id to be a string, got {type(id)}: {id!r}")
-            id = ascii_escaped(id)
+            if not isinstance(id, str) and id is not HIDDEN_PARAM:
+                raise TypeError(
+                    "Expected id to be a string or a `pytest.HIDDEN_PARAM` sentinel, "
+                    f"got {type(id)}: {id!r}",
+                )
         return cls(values, marks, id)
 
     @classmethod
     def extract_from(
         cls,
-        parameterset: Union["ParameterSet", Sequence[object], object],
+        parameterset: ParameterSet | Sequence[object] | object,
         force_tuple: bool = False,
-    ) -> "ParameterSet":
+    ) -> ParameterSet:
         """Extract from an object or objects.
 
         :param parameterset:
@@ -111,7 +150,6 @@ class ParameterSet(NamedTuple):
             Enforce tuple wrapping so single argument tuple values
             don't get decomposed and break tests.
         """
-
         if isinstance(parameterset, cls):
             return parameterset
         if force_tuple:
@@ -126,11 +164,11 @@ class ParameterSet(NamedTuple):
 
     @staticmethod
     def _parse_parametrize_args(
-        argnames: Union[str, Sequence[str]],
-        argvalues: Iterable[Union["ParameterSet", Sequence[object], object]],
+        argnames: str | Sequence[str],
+        argvalues: Iterable[ParameterSet | Sequence[object] | object],
         *args,
         **kwargs,
-    ) -> Tuple[Sequence[str], bool]:
+    ) -> tuple[Sequence[str], bool]:
         if isinstance(argnames, str):
             argnames = [x.strip() for x in argnames.split(",") if x.strip()]
             force_tuple = len(argnames) == 1
@@ -140,9 +178,9 @@ class ParameterSet(NamedTuple):
 
     @staticmethod
     def _parse_parametrize_parameters(
-        argvalues: Iterable[Union["ParameterSet", Sequence[object], object]],
+        argvalues: Iterable[ParameterSet | Sequence[object] | object],
         force_tuple: bool,
-    ) -> List["ParameterSet"]:
+    ) -> list[ParameterSet]:
         return [
             ParameterSet.extract_from(x, force_tuple=force_tuple) for x in argvalues
         ]
@@ -150,12 +188,12 @@ class ParameterSet(NamedTuple):
     @classmethod
     def _for_parametrize(
         cls,
-        argnames: Union[str, Sequence[str]],
-        argvalues: Iterable[Union["ParameterSet", Sequence[object], object]],
+        argnames: str | Sequence[str],
+        argvalues: Iterable[ParameterSet | Sequence[object] | object],
         func,
         config: Config,
         nodeid: str,
-    ) -> Tuple[Sequence[str], List["ParameterSet"]]:
+    ) -> tuple[Sequence[str], list[ParameterSet]]:
         argnames, force_tuple = cls._parse_parametrize_args(argnames, argvalues)
         parameters = cls._parse_parametrize_parameters(argvalues, force_tuple)
         del argvalues
@@ -185,33 +223,39 @@ class ParameterSet(NamedTuple):
             # parameter set with NOTSET values, with the "empty parameter set" mark applied to it.
             mark = get_empty_parameterset_mark(config, argnames, func)
             parameters.append(
-                ParameterSet(values=(NOTSET,) * len(argnames), marks=[mark], id=None)
+                ParameterSet(
+                    values=(NOTSET,) * len(argnames), marks=[mark], id="NOTSET"
+                )
             )
         return argnames, parameters
 
 
 @final
-@attr.s(frozen=True, init=False, auto_attribs=True)
+@dataclasses.dataclass(frozen=True)
 class Mark:
+    """A pytest mark."""
+
     #: Name of the mark.
     name: str
     #: Positional arguments of the mark decorator.
-    args: Tuple[Any, ...]
+    args: tuple[Any, ...]
     #: Keyword arguments of the mark decorator.
     kwargs: Mapping[str, Any]
 
     #: Source Mark for ids with parametrize Marks.
-    _param_ids_from: Optional["Mark"] = attr.ib(default=None, repr=False)
+    _param_ids_from: Mark | None = dataclasses.field(default=None, repr=False)
     #: Resolved/generated ids with parametrize Marks.
-    _param_ids_generated: Optional[Sequence[str]] = attr.ib(default=None, repr=False)
+    _param_ids_generated: Sequence[str] | None = dataclasses.field(
+        default=None, repr=False
+    )
 
     def __init__(
         self,
         name: str,
-        args: Tuple[Any, ...],
+        args: tuple[Any, ...],
         kwargs: Mapping[str, Any],
-        param_ids_from: Optional["Mark"] = None,
-        param_ids_generated: Optional[Sequence[str]] = None,
+        param_ids_from: Mark | None = None,
+        param_ids_generated: Sequence[str] | None = None,
         *,
         _ispytest: bool = False,
     ) -> None:
@@ -227,7 +271,7 @@ class Mark:
     def _has_param_ids(self) -> bool:
         return "ids" in self.kwargs or len(self.args) >= 4
 
-    def combined_with(self, other: "Mark") -> "Mark":
+    def combined_with(self, other: Mark) -> Mark:
         """Return a new Mark which is a combination of this
         Mark and another Mark.
 
@@ -239,7 +283,7 @@ class Mark:
         assert self.name == other.name
 
         # Remember source of ids with parametrize Marks.
-        param_ids_from: Optional[Mark] = None
+        param_ids_from: Mark | None = None
         if self.name == "parametrize":
             if other._has_param_ids():
                 param_ids_from = other
@@ -261,14 +305,14 @@ class Mark:
 Markable = TypeVar("Markable", bound=Union[Callable[..., object], type])
 
 
-@attr.s(init=False, auto_attribs=True)
+@dataclasses.dataclass
 class MarkDecorator:
     """A decorator for applying a mark on test functions and classes.
 
     ``MarkDecorators`` are created with ``pytest.mark``::
 
-        mark1 = pytest.mark.NAME              # Simple MarkDecorator
-        mark2 = pytest.mark.NAME(name1=value) # Parametrized MarkDecorator
+        mark1 = pytest.mark.NAME  # Simple MarkDecorator
+        mark2 = pytest.mark.NAME(name1=value)  # Parametrized MarkDecorator
 
     and can then be applied as decorators to test functions::
 
@@ -310,7 +354,7 @@ class MarkDecorator:
         return self.mark.name
 
     @property
-    def args(self) -> Tuple[Any, ...]:
+    def args(self) -> tuple[Any, ...]:
         """Alias for mark.args."""
         return self.mark.args
 
@@ -324,7 +368,7 @@ class MarkDecorator:
         """:meta private:"""
         return self.name  # for backward-compat (2.4.1 had this attr)
 
-    def with_args(self, *args: object, **kwargs: object) -> "MarkDecorator":
+    def with_args(self, *args: object, **kwargs: object) -> MarkDecorator:
         """Return a MarkDecorator with extra arguments added.
 
         Unlike calling the MarkDecorator, with_args() can be used even
@@ -337,11 +381,11 @@ class MarkDecorator:
     # return type. Not much we can do about that. Thankfully mypy picks
     # the first match so it works out even if we break the rules.
     @overload
-    def __call__(self, arg: Markable) -> Markable:  # type: ignore[misc]
+    def __call__(self, arg: Markable) -> Markable:  # type: ignore[overload-overlap]
         pass
 
     @overload
-    def __call__(self, *args: object, **kwargs: object) -> "MarkDecorator":
+    def __call__(self, *args: object, **kwargs: object) -> MarkDecorator:
         pass
 
     def __call__(self, *args: object, **kwargs: object):
@@ -349,17 +393,22 @@ class MarkDecorator:
         if args and not kwargs:
             func = args[0]
             is_class = inspect.isclass(func)
-            if len(args) == 1 and (istestfunc(func) or is_class):
-                store_mark(func, self.mark)
+            # For staticmethods/classmethods, the marks are eventually fetched from the
+            # function object, not the descriptor, so unwrap.
+            unwrapped_func = func
+            if isinstance(func, (staticmethod, classmethod)):
+                unwrapped_func = func.__func__
+            if len(args) == 1 and (istestfunc(unwrapped_func) or is_class):
+                store_mark(unwrapped_func, self.mark, stacklevel=3)
                 return func
         return self.with_args(*args, **kwargs)
 
 
 def get_unpacked_marks(
-    obj: Union[object, type],
+    obj: object | type,
     *,
     consider_mro: bool = True,
-) -> List[Mark]:
+) -> list[Mark]:
     """Obtain the unpacked marks that are stored on an object.
 
     If obj is a class and consider_mro is true, return marks applied to
@@ -370,7 +419,9 @@ def get_unpacked_marks(
         if not consider_mro:
             mark_lists = [obj.__dict__.get("pytestmark", [])]
         else:
-            mark_lists = [x.__dict__.get("pytestmark", []) for x in obj.__mro__]
+            mark_lists = [
+                x.__dict__.get("pytestmark", []) for x in reversed(obj.__mro__)
+            ]
         mark_list = []
         for item in mark_lists:
             if isinstance(item, list):
@@ -387,7 +438,7 @@ def get_unpacked_marks(
 
 
 def normalize_mark_list(
-    mark_list: Iterable[Union[Mark, MarkDecorator]]
+    mark_list: Iterable[Mark | MarkDecorator],
 ) -> Iterable[Mark]:
     """
     Normalize an iterable of Mark or MarkDecorator objects into a list of marks
@@ -399,16 +450,22 @@ def normalize_mark_list(
     for mark in mark_list:
         mark_obj = getattr(mark, "mark", mark)
         if not isinstance(mark_obj, Mark):
-            raise TypeError(f"got {repr(mark_obj)} instead of Mark")
+            raise TypeError(f"got {mark_obj!r} instead of Mark")
         yield mark_obj
 
 
-def store_mark(obj, mark: Mark) -> None:
+def store_mark(obj, mark: Mark, *, stacklevel: int = 2) -> None:
     """Store a Mark on an object.
 
     This is used to implement the Mark declarations/decorators correctly.
     """
     assert isinstance(mark, Mark), mark
+
+    from ..fixtures import getfixturemarker
+
+    if getfixturemarker(obj) is not None:
+        warnings.warn(MARKED_FIXTURE, stacklevel=stacklevel)
+
     # Always reassign name to avoid updating pytestmark in a reference that
     # was only borrowed.
     obj.pytestmark = [*get_unpacked_marks(obj, consider_mro=False), mark]
@@ -417,59 +474,52 @@ def store_mark(obj, mark: Mark) -> None:
 # Typing for builtin pytest marks. This is cheating; it gives builtin marks
 # special privilege, and breaks modularity. But practicality beats purity...
 if TYPE_CHECKING:
-    from _pytest.scope import _ScopeName
 
     class _SkipMarkDecorator(MarkDecorator):
-        @overload  # type: ignore[override,misc,no-overload-impl]
-        def __call__(self, arg: Markable) -> Markable:
-            ...
+        @overload  # type: ignore[override,no-overload-impl]
+        def __call__(self, arg: Markable) -> Markable: ...
 
         @overload
-        def __call__(self, reason: str = ...) -> "MarkDecorator":
-            ...
+        def __call__(self, reason: str = ...) -> MarkDecorator: ...
 
     class _SkipifMarkDecorator(MarkDecorator):
         def __call__(  # type: ignore[override]
             self,
-            condition: Union[str, bool] = ...,
-            *conditions: Union[str, bool],
+            condition: str | bool = ...,
+            *conditions: str | bool,
             reason: str = ...,
-        ) -> MarkDecorator:
-            ...
+        ) -> MarkDecorator: ...
 
     class _XfailMarkDecorator(MarkDecorator):
-        @overload  # type: ignore[override,misc,no-overload-impl]
-        def __call__(self, arg: Markable) -> Markable:
-            ...
+        @overload  # type: ignore[override,no-overload-impl]
+        def __call__(self, arg: Markable) -> Markable: ...
 
         @overload
         def __call__(
             self,
-            condition: Union[str, bool] = ...,
-            *conditions: Union[str, bool],
+            condition: str | bool = False,
+            *conditions: str | bool,
             reason: str = ...,
             run: bool = ...,
-            raises: Union[Type[BaseException], Tuple[Type[BaseException], ...]] = ...,
+            raises: None
+            | type[BaseException]
+            | tuple[type[BaseException], ...]
+            | AbstractRaises[BaseException] = ...,
             strict: bool = ...,
-        ) -> MarkDecorator:
-            ...
+        ) -> MarkDecorator: ...
 
     class _ParametrizeMarkDecorator(MarkDecorator):
         def __call__(  # type: ignore[override]
             self,
-            argnames: Union[str, Sequence[str]],
-            argvalues: Iterable[Union[ParameterSet, Sequence[object], object]],
+            argnames: str | Sequence[str],
+            argvalues: Iterable[ParameterSet | Sequence[object] | object],
             *,
-            indirect: Union[bool, Sequence[str]] = ...,
-            ids: Optional[
-                Union[
-                    Iterable[Union[None, str, float, int, bool]],
-                    Callable[[Any], Optional[object]],
-                ]
-            ] = ...,
-            scope: Optional[_ScopeName] = ...,
-        ) -> MarkDecorator:
-            ...
+            indirect: bool | Sequence[str] = ...,
+            ids: Iterable[None | str | float | int | bool]
+            | Callable[[Any], object | None]
+            | None = ...,
+            scope: _ScopeName | None = ...,
+        ) -> MarkDecorator: ...
 
     class _UsefixturesMarkDecorator(MarkDecorator):
         def __call__(self, *fixtures: str) -> MarkDecorator:  # type: ignore[override]
@@ -489,9 +539,10 @@ class MarkGenerator:
 
          import pytest
 
+
          @pytest.mark.slowtest
          def test_function():
-            pass
+             pass
 
     applies a 'slowtest' :class:`Mark` on ``test_function``.
     """
@@ -507,8 +558,8 @@ class MarkGenerator:
 
     def __init__(self, *, _ispytest: bool = False) -> None:
         check_ispytest(_ispytest)
-        self._config: Optional[Config] = None
-        self._markers: Set[str] = set()
+        self._config: Config | None = None
+        self._markers: set[str] = set()
 
     def __getattr__(self, name: str) -> MarkDecorator:
         """Generate a new :class:`MarkDecorator` with the given name."""
@@ -542,9 +593,9 @@ class MarkGenerator:
                     fail(f"Unknown '{name}' mark, did you mean 'parametrize'?")
 
                 warnings.warn(
-                    "Unknown pytest.mark.%s - is this a typo?  You can register "
+                    f"Unknown pytest.mark.{name} - is this a typo?  You can register "
                     "custom marks to avoid this warning - for details, see "
-                    "https://docs.pytest.org/en/stable/how-to/mark.html" % name,
+                    "https://docs.pytest.org/en/stable/how-to/mark.html",
                     PytestUnknownMarkWarning,
                     2,
                 )
@@ -557,9 +608,9 @@ MARK_GEN = MarkGenerator(_ispytest=True)
 
 @final
 class NodeKeywords(MutableMapping[str, Any]):
-    __slots__ = ("node", "parent", "_markers")
+    __slots__ = ("_markers", "node", "parent")
 
-    def __init__(self, node: "Node") -> None:
+    def __init__(self, node: Node) -> None:
         self.node = node
         self.parent = node.parent
         self._markers = {node.name: True}
@@ -579,15 +630,13 @@ class NodeKeywords(MutableMapping[str, Any]):
     # below and use the collections.abc fallback, but that would be slow.
 
     def __contains__(self, key: object) -> bool:
-        return (
-            key in self._markers
-            or self.parent is not None
-            and key in self.parent.keywords
+        return key in self._markers or (
+            self.parent is not None and key in self.parent.keywords
         )
 
     def update(  # type: ignore[override]
         self,
-        other: Union[Mapping[str, Any], Iterable[Tuple[str, Any]]] = (),
+        other: Mapping[str, Any] | Iterable[tuple[str, Any]] = (),
         **kwds: Any,
     ) -> None:
         self._markers.update(other)
